import asyncio

from functools import partial
from kafka import KafkaConsumer as _KC


class KafkaConsumer(_KC):
    callback = None

    def _consume(self):
        try:
            return self.__next__()
        except StopIteration as e:
            return None

    async def consume(self):
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(None, self._consume)

    @classmethod
    def from_etcd(cls, callback=None):
        cls.callback = cls.callback or callback
        return partial(cls,
                       bootstrap_servers=callback(),
                       consumer_timeout_ms=1000)


import re
import getpass

from star_builder.build.tasks import Drivable, Project as _Project, ModuleTask
from os.path import join, exists, abspath, dirname, isdir, basename


__all__ = ["Project", "Backend", "Service"]


class Project(_Project):
    """
    项目
    """
    @classmethod
    def enrich_parser(cls, sub_parser):
        super().enrich_parser(sub_parser)
        reg = "hub.yiducloud.cn/library"
        sub_parser.add_argument(
            "-r", "--registry", help=f"镜像仓库地址 默认：{reg}", default=reg)
        sub_parser.add_argument("-v", "--version", help="版本", default="0.1.0")
        index = "http://devpi.intra.yiducloud.cn/root/yiducloud/+simple/"
        sub_parser.add_argument(
            "-i", "--index-url", help=f"Pip源 默认：{index}", default=index)
        t = "devpi.intra.yiducloud.cn"
        sub_parser.add_argument(
            "-th", "--trusted-host", help=f"信任的域名 默认：{t}", default=t)
        user = getpass.getuser()
        sub_parser.add_argument(
            "-a", "--author", help=f"作者 默认：{user}", default=user)


class Service(ModuleTask):
    """
    服务
    """

    def enrich_kwargs(self, words):
        super().enrich_kwargs(words)
        father = None

        if exists("__init__.py"):
            regex = re.compile(r"class\s+(\w*?Service)\(\w*Service\):")
            mth = regex.search(open("__init__.py").read())
            if mth:
                father = mth.group(1)
        self.kwargs["father"] = father or "Service"

    @classmethod
    def enrich_parser(cls, sub_parser):
        sub_parser.add_argument("name", nargs="+", help="服务模块名称")


class Backend(Drivable, ModuleTask):
    """
    后台服务驱动
    """

    def validate_name(self, name):
        words = re.findall(r"([A-Za-z0-9]+)", name)
        if words[0][0].isdigit():
            words.insert(0, "backend")
        self.kwargs["name"] = name
        return words

    def enrich_kwargs(self, name):
        super(Backend, self).enrich_kwargs(name)
        self.kwargs["dirname"] = "backend"
        if self.kwargs["module"] == "paas_star.backend" and \
                self.kwargs["cls"] == "RestfulApi":
            self.kwargs["base"] = ""
        else:
            self.kwargs["base"] = "EtcdMixin"

    @classmethod
    def enrich_parser(cls, sub_parser):
        super().enrich_parser(sub_parser)
        sub_parser.add_argument("name", nargs="+", help="后台服务名称")
        sub_parser.add_argument(
            "-d", "--driver",
            default="paas_star.backend:RestfulApi",
            help="驱动程序基类，default: paas_star.backend:RestfulApi")




from star_builder import Component

from .iam import Iam
from ..routing.routing import Routing


class IamComponent(Component):

    def resolve(self, routing: Routing) -> Iam:
        return Iam(routing)



import typing
from apistar.http import QueryString
from boto.s3.connection import OrdinaryCallingFormat

from toolkit.frozen import FrozenSettings
from star_builder import Component

from .dummy_s3 import DummyS3
from ..mongodb.mongodb import Mongodb
from .s3 import S3 as S3Connection

S3 = typing.TypeVar('S3', S3Connection, DummyS3)


class S3Component(Component):

    async def resolve(self, settings: FrozenSettings,
                mongodb: Mongodb,
                project: QueryString) -> S3:
        if settings.PAAS_CLUSTER in settings.get(
                "PUBLIC_CLUSTER", "adr,research").split(","):
            col = mongodb[settings.S3_CONFIG_DB][settings.S3_CONFIG_COLLECTION]
            keys = await col.find_one(
                {
                    "project": project or "pangu"
                },
                {
                    "aws_access_key_id": 1,
                    "aws_secret_access_key": 1,
                    "_id": 0
                })

            if not keys:
                keys = {"aws_access_key_id": settings.ACCESS_KEY,
                        "aws_secret_access_key": settings.SECRET_KEY}
            # 公有云
            return S3Connection(
                **keys,
                host=settings.S3_HOST,
                is_secure=False,
                calling_format=OrdinaryCallingFormat(),
            )
        else:
            # 私有云
            return DummyS3(mongodb)



from random import choice
from urllib.parse import urljoin
from toolkit import cache_property
from archsdk import get_service

from ..logger import get_logger


class Manager(object):

    def __init__(self, callback):
        self.callback = callback
        self._instance = callback()
        self.reload = False

    def __call__(self, reload=False):
        self.reload = reload
        return self

    def __enter__(self):
        if self.reload:
            self._instance = self.callback()
        return self._instance

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type is not None and not self.reload:
            self._instance = self.callback()
        return exc_type is None


class EtcdMixin(object):
    callback = None

    @classmethod
    def from_etcd(cls, service=None):
        if service:
            cls.callback = lambda: choice(get_service(service))
        return cls(**cls.callback())


class LoggerMixin(object):

    @cache_property
    def logger(self):
        """
        可以选择覆盖这个属性
        :return:
        """
        return get_logger("unset")

    def set_logger(self, logger):
        self.logger = logger


class RestfulApi(EtcdMixin, LoggerMixin):

    def __init__(self, host, port):
        self.host = host
        self.port = port

    def url(self, path):
        return urljoin(f'http://{self.host}:{self.port}', path)



# -*- coding:utf-8 -*-
"""
使用gridfs 模拟s3服务部分model和接口
"""
import motor


class DummyS3(object):
    """
        dummy s3 conn
    """
    def __getattr__(self, item):
        raise AttributeError("DummyS3 doesn't support %s" % item)

    def __init__(self, mongo):
        self.mongo = mongo

    async def get_bucket(self, bucket):
        return Bucket(bucket, mongo=self.mongo)

    async def lookup(self, bucket):
        return Bucket(bucket, mongo=self.mongo)


class Bucket(object):
    """
    dummy s3 bucket
    """
    def __init__(self, name, mongo):
        self.mongo = mongo
        self.name = name

    async def delete_key(self, name):
        db = motor.MotorGridFS(self.mongo["s3"], collection=self.name)
        f = await db.find_one({"filename": name})
        return await db.delete(f.name)

    async def new_key(self, name):
        return Key(self.name, name, self.mongo)

    async def get_key(self, name):
        return Key(self.name, name, self.mongo)


class Key(object):
    """
    dummy s3 key
    """
    def __init__(self, bucket_name, name, mongo):
        self.bucket_name = bucket_name
        self.name = name
        self.mongo = mongo

    async def set_contents_from_string(self, content):
        db = motor.MotorGridFS(self.mongo["s3"], collection=self.bucket_name)
        await db.put(content, filename=self.name)

    async def set_canned_acl(self, arg):
        pass

    async def get_contents_as_string(self):
        db = motor.MotorGridFS(self.mongo["s3"], collection=self.bucket_name)
        doc = await db.find_one({"filename": self.name})
        return await doc.read()


import asyncio
from boto import connect_s3


class S3(object):

    def __init__(self, *args, **kwargs):
        self.conn = connect_s3(*args, **kwargs)

    def __getattr__(self, item):
        return getattr(self.conn, item)

    async def get_bucket(self, bucket):
        loop = asyncio.get_event_loop()
        return Bucket(
            await loop.run_in_executor(None, self.conn.get_bucket, bucket))

    async def lookup(self, bucket):
        loop = asyncio.get_event_loop()
        return Bucket(
            await loop.run_in_executor(None, self.conn.lookup, bucket))


class Bucket(object):
    """
    dummy s3 bucket
    """
    def __init__(self, bucket):
        self.bucket = bucket

    async def delete_key(self, name):
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(None, self.bucket.delete_key, name)

    async def new_key(self, name):
        return Key(self.bucket.new_key(name))

    async def get_key(self, name):
        loop = asyncio.get_event_loop()
        return Key(await loop.run_in_executor(None, self.bucket.get_key, name))


class Key(object):
    """
    dummy s3 key
    """
    def __init__(self, key):
        self.key = key

    async def set_contents_from_string(self, content):
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(
            None, self.key.set_contents_from_string, content)

    async def set_canned_acl(self, arg):
        pass

    async def get_contents_as_string(self):
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(
            None, self.key.get_contents_as_string)



from toolkit import _find_caller_name
from star_builder import Service

from .session import Session
from ..logger import get_logger


class Repository(Service):

    def __init__(self, *args, **kwargs):
        name = kwargs.pop("name", _find_caller_name())
        self.logger = get_logger(name)

    def resolve(self, session: Session):
        """
        每次请求都更新logger.user
        :param session:
        :return:
        """
        self.logger.user = session.user



import typing

from functools import partial
from star_builder import Component

from .. import Manager
from .mongodb import Mongodb

SERVICE_NAME = "mongodb"
MongodbManger = typing.NewType("MongodbManger", Manager)


class MongodbMangerComponent(Component):

    def resolve(self) -> MongodbManger:
        return MongodbManger(
            Manager(partial(Mongodb.from_etcd, SERVICE_NAME)))


class MongodbComponent(Component):

    def resolve(self) -> Mongodb:
        return Mongodb.from_etcd(SERVICE_NAME)



import json
import aiohttp

from collections import namedtuple
from apistar.exceptions import HTTPException, BadRequest


class UnauthorizedException(HTTPException):
    default_status_code = 401


class IamClient(object):

    def __init__(self, routing):
        self.routing = routing

    async def who(self, session):
        try:
            async with aiohttp.ClientSession(
                    cookies={"sso_usession": session}) as ses:
                async with ses.get(self.routing.url("/sso/oauth/iam/who"),
                                   headers={'X-Session-Info': session}) as resp:
                    if resp.status in [400, 401, 404]:
                        return None
                    buffer = await resp.read()
        except Exception as e:
            raise BadRequest(detail=str(e)) from e

        if resp.status != 200:
            raise HTTPException(detail=buffer, status_code=resp.status)
        user = json.loads(buffer).get("value")
        if not user:
            raise UnauthorizedException(f"用户不存在 [{session}]")
        return namedtuple("User", user.keys())(**user)



import typing

from functools import partial
from star_builder import Component

from .. import Manager
from .routing import Routing

SERVICE_NAME = "routing"

RoutingManger = typing.NewType("RoutingManger", Manager)


class RoutingMangerComponent(Component):

    def resolve(self) -> RoutingManger:
        return RoutingManger(
            Manager(partial(Routing.from_etcd, SERVICE_NAME)))


class MongoComponent(Component):

    def resolve(self) -> Routing:
        return Routing.from_etcd(SERVICE_NAME)



import os
import sys
import logging

from logging import handlers
from functools import wraps


def get_logger(name="root"):
    return Logger(name)


class Logger(object):
    """
    logger的实现
    """
    format_string = "{asctime} {level} {logger}:" \
                    " env: {env}, user: {user}, {message}"

    def __init__(self, name):
        self.name = name
        self.logger = logging.getLogger(self.name)
        self.logger.propagate = False
        self.json = False
        if os.getenv("LOG_FILE") in ["true", "True", "1"]:
            dictionary = os.getenv("LOG_DIR", "/tmp")
            filename = os.getenv("LOG_FILENAME", "test.log")
            os.makedirs(dictionary, exist_ok=True)
            file_handler = handlers.RotatingFileHandler(
                os.path.join(dictionary, filename),
                maxBytes=10*1024*1024,
                backupCount=7)
            self.set_handler(file_handler)
        else:
            self.set_handler(logging.StreamHandler(sys.stdout))
        self.user = None

    def set_handler(self, handler):
        handler.setLevel(logging.DEBUG)
        handler.setFormatter(self._get_formatter())
        if not self.logger.handlers:
            self.logger.addHandler(handler)

    def __getattr__(self, item):
        if item.upper() in logging._nameToLevel:
            func = getattr(self.logger, item)

            @wraps(func)
            def wrapper(*args, **kwargs):
                extra = kwargs.pop("extra", {})
                extra.setdefault("level", item.upper())
                extra.setdefault('env', os.getenv("PAAS_TARGET", "")),
                extra.setdefault('user', getattr(self.user, "name", None)),
                extra.setdefault("logger", self.name)
                kwargs["extra"] = extra
                return func(*args, **kwargs)
            return wrapper

        raise AttributeError

    def _get_formatter(self):
        if self.json:
            # 目前没有安装此依赖
            from pythonjsonlogger import jsonlogger
            return jsonlogger.JsonFormatter()
        else:
            return logging.Formatter(self.format_string, style="{")


from apistar import http
from star_builder import Component, Cookie

from ..backend.iam import Iam


class Session(object):
    def __init__(self, sid: str, iam: Iam):
        self.sid = sid
        self.iam = iam
        self.user = None
    
    async def get_user(self):
        self.user = self.sid and (await self.iam.who(self.sid))


class SessionComponent(Component):

    async def resolve(self, iam: Iam,
                x_session_info: http.Header,
                x_user_info: http.Header,
                session: Cookie,
                token: Cookie) -> Session:

        sid = x_session_info or x_user_info or session or token
        session = Session(sid, iam)
        await session.get_user()
        return session



import typing

from functools import partial
from archsdk import get_service
from star_builder import Component

from .. import Manager
from .kafka_consumer import KafkaConsumer
from .kafka_producer import KafkaProducer

SERVICE = "kafka"
KafkaConsumerManger = typing.NewType("KafkaConsumerManger", Manager)
KafkaProducerManger = typing.NewType("KafkaProducerManger", Manager)


class CallbackMixin(object):

    def callback(self):
        return [c["host"] + ":" + str(c["port"]) for c in get_service(SERVICE)]


class KafkaConsumerComponent(Component, CallbackMixin):

    def resolve(self) -> KafkaConsumer:
        client_warpper = partial(
            KafkaConsumer,
            bootstrap_servers=self.callback(),
            consumer_timeout_ms=1000)
        return client_warpper


class KafkaProducerComponent(Component, CallbackMixin):

    def resolve(self) -> KafkaProducer:
        return KafkaProducer(bootstrap_servers=self.callback(), retries=3)


class KafkaConsumerManagerComponent(Component, CallbackMixin):

    def resolve(self) -> KafkaConsumerManger:
        return KafkaConsumerManger(
            Manager(partial(KafkaConsumer.from_etcd, callback=self.callback)))


class KafkaProducerManagerComponent(Component, CallbackMixin):

    def resolve(self) -> KafkaProducerManger:
        return KafkaProducerManger(
            Manager(partial(KafkaConsumer.from_etcd, callback=self.callback)))



from os.path import join, abspath, dirname
from star_builder.build import Command, find_tasks, Task

from .tasks import *


class PaasCommand(Command):

    def __init__(self, tasks):
        for name, prop in globals().items():
            if prop is not Task and \
                    isinstance(prop, type) and \
                    issubclass(prop, Task):
                tasks[name.lower()] = prop
        super(PaasCommand, self).__init__(tasks)
        self.templates.insert(
            1, join(abspath(dirname(dirname(__file__))), "templates"))


def main():
    PaasCommand(find_tasks()).create()



import typing

from star_builder import Component
from apistar.http import QueryParams, RequestData

Model = typing.NewType("model", dict)


class ModelComponent(Component):
    """
    聚合参数的基类
    """

    def resolve(self, *args, **kwargs) -> object:
        return super(ModelComponent, self).resolve()

    @staticmethod
    def gather(params: QueryParams, form: RequestData) -> Model:
        total_params = dict()
        if params:
            total_params.update(params)
        if form:
            total_params.update(form)
        return Model(total_params)



# coding: utf-8

from setuptools import find_packages, setup


def install_requires():
    """
    Return requires in requirements.txt
    :return:
    """
    try:
        with open("requirements.txt") as f:
            return [line.strip() for line in f.readlines() if line.strip()]
    except OSError:
        return []

setup(
    name='paas-star',
    version='0.2.2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=install_requires(),
    author='shichao.ma',
    author_email='shichao.ma@yiducloud.cn',
    description='''package description here''',
    entry_points={
        'console_scripts': [
            'paas-create = paas_star:main',
        ],
    },
    keywords='',
    
)


from paas_star import Type
import asyncio
from star_builder import validators
import uuid
import datetime

class Book(Type):
    id = validators.String(default=uuid.uuid1, format="UUID")
    created_at =validators.DateTime(default=datetime.datetime.now)
    name = validators.String()

    async def load(self):
        return {"name": "张三", }


async def test_model():
    b = Book(name="114")
    #b.format()
    print(b["id"])
    print(b["created_at"])
    print(await b.id)
    created_at = await b.created_at
    print(created_at)
    print(type(created_at))
    print(await b.name)

    b.format()
    b["id"] = "aaaa"
    print(b["id"])


def main():
    loop = asyncio.get_event_loop()
    loop.run_until_complete(test_model())


main()


import asyncio

from functools import partial
from kafka import KafkaProducer as _KP


class KafkaProducer(_KP):
    callback = None

    def produce_callback(self, waiter, value):
        waiter.set_result(value)

    def produce_errback(self, waiter, value):
        waiter.set_result(value)

    async def produce(self, topic, message):
        loop = asyncio.get_event_loop()
        waiter = loop.create_future()
        future = super(KafkaProducer, self).send(topic, message.encode())
        future.add_callback(self.produce_callback, waiter)
        future.add_errback(self.produce_errback, waiter)
        await waiter
        return future.get()

    @classmethod
    def from_etcd(cls, callback=None):
        cls.callback = cls.callback or callback
        return partial(cls,
                       bootstrap_servers=callback(),
                       retries=3)



import typing

from functools import partial
from star_builder import Component

from .mongo import Mongo
from .. import Manager

SERVICE_NAME = "MONGO"
MongoManger = typing.NewType("MongoManger", Manager)


class MongoMangerComponent(Component):

    def resolve(self) -> MongoManger:
        return MongoManger(
            Manager(partial(Mongo.from_etcd, SERVICE_NAME)))


class MongoComponent(Component):

    def resolve(self) -> Mongo:
        return Mongo.from_etcd(SERVICE_NAME)
